SCREEN_SIZE =  (640, 480)

import pygame
from pygame.locals import *
from gameobjects.vector3 import Vector3
from gameobjects.vector2 import Vector2
from gameobjects.matrix import Matrix
from math import radians
from random import randint

from subpixelsurface import SubPixelSurface

try:
    import psyco
    psyco.full()
except ImportError:
    print "Runs better with Psyco!"
    pass

def run():
    
    pygame.init()
    screen = pygame.display.set_mode(SCREEN_SIZE, FULLSCREEN)
    back = pygame.image.load("lissaback.jpg").convert()
    ball = pygame.image.load("ball.png").convert_alpha()
    ball_sub = SubPixelSurface(ball, x_level=4)
    
    viewing_distance = 400.    
    
    def new_figure():
    
        arms = [ (150., Vector3(0., 0., 1.)) ]
        
        for n in xrange(randint(1, 5)):
            rand_length = float(randint(50, 120))
            rand_rotation = Vector3(float(randint(0, 7)), float(randint(0, 7)), float(randint(0, 7)))
            arms.append((rand_length, rand_rotation))
                         
        points = []
                    
        for t in xrange(0, 360):
            
            a = radians(float(t))
            
            mat = Matrix.identity()
            for arm in arms:
            
                length, rot_speed = arm
                m = Matrix.xyz_rotation( *(rot_speed * a) )
                m *= Matrix.translate(0, length, 0)                
                mat *= m
            points.append( mat.transform((0., 0., 0.)) )
             
        return points   
    
    points = new_figure()
    
    camera_position = Vector3(0, 0, -SCREEN_SIZE[1]*1.5)
        
        
    center_x, center_y = SCREEN_SIZE
    center_x /= 2
    center_y /= 2
    
    clock = pygame.time.Clock()
        
    def point_z(p):
        return -p.z
        
    t = 0.
    
    down_pos = Vector2()
    mouse_down = False
    
    mouse_rotation = Vector3()
    auto_rotation = Vector3()
    auto_rotation_speed = Vector3(0, radians(20.), 0)
    
    while True:
        
        for event in pygame.event.get():
            if event.type == QUIT:
                return
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    return
                if event.key == K_SPACE:
                    points = new_figure()
            if event.type == MOUSEBUTTONDOWN:                
                mouse_down=True
            elif event.type == MOUSEBUTTONUP:
                mouse_down = False
            elif event.type == MOUSEMOTION:
                if mouse_down:
                    rel = Vector2(*event.rel) / 200.
                    mouse_rotation.y += rel.x
                    mouse_rotation.x += rel.y
            
                            
        screen.blit(back, (0, 0))
        
        time_passed = clock.tick()
        time_passed_seconds = time_passed / 1000.        
    
        auto_rotation += auto_rotation_speed * time_passed_seconds
        
        
        rotation_matrix = Matrix.xyz_rotation(*(auto_rotation + mouse_rotation))
    
        points_2d = []
        
        transformed_points = map(rotation_matrix.transform, points)
        
        for point in sorted(transformed_points, key=point_z):
            
            x, y, z = point - camera_position
            
            x = center_x + x * viewing_distance / z
            y = center_y + y * viewing_distance / z
            screen.blit(ball_sub.at(x, y), (x, y))
                        
            
        pygame.display.update()
        
run()